**Free
//
// FindFil_Ex
// ----------
// Example program for the FindFiles() function.
//
// Jesper Wachs, Version 2, August 2018.
//   Added parameter for indicating if folder names should be matched
//   against file specs as well.
//   Converted to Totally Free RPG.
//
// Jesper Wachs, Version 1, August 2016.
//
// To successful compile this program, you must have downloaded and
// compiled these two functions from my website:
// - LUCase
// - SplitFN
//
// You will find then in the 'ILE RPG Stuff' section.
//
// The source member are delivered 'as is'.
//
// Neither Jesper Wachs nor anyone else who has been involved in
// the creation, production or delivery of this product shall be
// liable for any direct, indirect, consequential or incidental
// damages (including damages for loss of business profits,
// business interruption, loss of business information, and the
// like) arising out of the use or inability to use such product
// even if Jesper Wachs has been advised of the possibility of
// such damages.
//
// Copymember for imported functions.
// ----------------------------------
/copy qrpglesrc,findfil_pr

// Prototypes for local functions.
// -------------------------------
Dcl-PR FileFound Ind;
  iPath Char(250) Value;
  iFileName Char(100) Value;
  iIsDir Ind Value;
  iStatDS Value Like(StatDS);
  iErrNo Int(10:0) Value;
End-PR;

Dcl-PR CnvUTimeToTS TimeStamp;
  sTime Uns(10:0) Const;
End-PR;

// Work fields.
// ------------
Dcl-DS StatDS Len(128);
  st_mode Uns(10);
  st_ino Uns(10);
  st_nlink Uns(5);
  reserved1 Char(2);
  st_uid Uns(10);
  st_gid Uns(10);
  st_size Uns(10);
  st_atime Uns(10);
  st_mtime Uns(10);
  st_ctime Uns(10);
  st_dev Uns(10);
  st_blksize Int(10:0);
  st_allocsize Int(10:0);
  st_objtype Char(10);
  reserved2 Char(2);
  st_codepage Uns(5);
  st_reserved1 Char(62);
  st_ino_genid Uns(10);
End-DS;

Dcl-S Msg Char(50);

// Main line of program.
// ---------------------
// This is the call to the actual file search function.

  If FindFiles('/qsys.lib/jwtools.lib':
               '*.pgm': *On:
               *Off:
               %PAddr(FileFound)) = *On;
     Msg = 'Search was terminated';
  Else;
     Msg = 'Search was successful';
  EndIf;

  Dsply Msg;

  *InLR = *On;
  Return;

//
// FileFound
// ---------
// This function is a 'callback' function. It is called every time
// a file is found that matches the search mask.
//
Dcl-Proc FileFound;
Dcl-PI FileFound Ind;
  iPath Char(250) Value;
  iFileName Char(100) Value;
  iIsDir Ind Value;
  iStatDS Value Like(StatDS);
  iErrNo Int(10:0) Value;
End-PI;

// Prototype for API that returns the text for an error number.

Dcl-PR StrError Pointer ExtProc('strerror');
  ErrNum Int(10:0) Value;
End-PR;

// Stat data structure parsed on the call. For the meaning of each
// field, search the net :-)

Dcl-DS StatDS Len(128);
  st_mode Uns(10);
  st_ino Uns(10);
  st_nlink Uns(5);
  reserved1 Char(2);
  st_uid Uns(10);
  st_gid Uns(10);
  st_size Uns(10);
  st_atime Uns(10);
  st_mtime Uns(10);
  st_ctime Uns(10);
  st_dev Uns(10);
  st_blksize Int(10:0);
  st_allocsize Int(10:0);
  st_objtype Char(10);
  reserved2 Char(2);
  st_codepage Uns(5);
  st_reserved1 Char(62);
  st_ino_genid Uns(10);
End-DS;

// Work fields

Dcl-S p_errmsg Pointer;
Dcl-S Msg Char(50);
Dcl-S Stamp TimeStamp;
Dcl-S ReturnCode Ind Inz(*Off);

// If an error occurred, then display that error.

  If iErrNo <> *Zeros;
     p_errmsg = StrError(iErrNo);
     Msg = 'Error reading ' +%TrimR(iPath) +
          ': ' + %Str(p_errmsg);
     Dsply Msg;
  Else;

     // In this example, we ignore the parent dir.

     If iFileName <> '..';

        If iIsDir = *On;
           Msg = 'Dir.:';
        Else;
           Msg = 'File:';
        EndIf;

        Msg = %TrimR(Msg) + ' '+ %TrimR(iPath) +
             %TrimR(iFileName);
        Dsply Msg;

        // As iStatDS is merly a string field, we must convert to StatDS
        // structure to get each field.

        StatDS = iStatDS;

        // Call convert routine to get a 'good old timestamp' of when the
        // file was last modified.

        Stamp = CnvUTimeToTS(st_mtime);
        Msg = 'Last modified: ' + %Char(Stamp);

        Dsply Msg;

     EndIf;
  EndIf;

  Return ReturnCode;

End-Proc FileFound;

// CnvUTimeToTS
// ------------
// Convert Unix time field into TimeStamp field taking into account
// the local time zone.

Dcl-Proc CnvUTimeToTS;
Dcl-PI CnvUTimeToTS TimeStamp;
  iTime Uns(10:0) Const;
End-PI;

Dcl-PR LocalTime Pointer ExtProc('localtime');
  TimeVal Pointer Value;
End-PR;

Dcl-DS tm Based(tmP);
  tm_sec Int(10:0);
  tm_min Int(10:0);
  tm_hour Int(10:0);
  tm_mday Int(10:0);
  tm_mon Int(10:0);
  tm_year Int(10:0);
  tm_wday Int(10:0);
  tm_yday Int(10:0);
  tm_isdst Int(10:0);
End-DS;

Dcl-S tmP Pointer;

Dcl-S wTime Uns(10:0);
Dcl-S nStamp Zoned(20:0);
Dcl-S oStamp TimeStamp Inz(z'0001-01-01-01.01.01.000000');

  wTime = iTime;
  tmP = LocalTime(%Addr(wTime));

  If tmP <> *Null;

     nStamp = tm_year + 1900;
     nStamp = nStamp * 100;
     nStamp = nStamp + tm_mon + 1;
     nStamp = nStamp * 100;
     nStamp = nStamp + tm_mday;
     nStamp = nStamp * 100;
     nStamp = nStamp + tm_hour;
     nStamp = nStamp * 100;
     nStamp = nStamp + tm_min;
     nStamp = nStamp * 100;
     nStamp = nStamp + tm_sec;
     nStamp = nStamp * 1000000;

     oStamp = %TimeStamp(nStamp: *ISO);

  EndIf;

  Return oStamp;

End-Proc CnvUTimeToTS;
