**Free
//
// FindFil_Pr
// ----------
// Prototypes for FindFiles() function.
//
// Jesper Wachs, Version 2, August 2018:
//   Added parameter for indicating if folder names should be matched
//   against file specs as well.
//   Converted to Totally Free RPG.
//
// Jesper Wachs, Version 1, August 2016.
//
// The source member are delivered 'as is'.
//
// Neither Jesper Wachs nor anyone else who has been involved in
// the creation, production or delivery of this product shall be
// liable for any direct, indirect, consequential or incidental
// damages (including damages for loss of business profits,
// business interruption, loss of business information, and the
// like) arising out of the use or inability to use such product
// even if Jesper Wachs has been advised of the possibility of
// such damages.
//
Dcl-PR FindFiles Ind;
  iStartPath Char(250) Value;
  iFileMask Char(25) Value;
  iMaskDir Ind Value;
  iDoSubDirs Ind Value;
  iFoundProc Pointer(*Proc) Value;
End-PR;
//
// The function FindFiles must be called with these parameters:
//
// FindFiles(
//           starting_path:    The path to start the search. This can
//                             be a IFS or QSYS path. FindFiles() work
//                             on both.
//           file_mask:        The mask which found files must match.
//                             You can pass these values to return all
//                             files/objects:
//                             '*.*'
//                             '*'
//                             ' '
//                             You can enter the mask in upper or
//                             lower case. FindFiles() is case
//                             insensitive. If you enter '*abc*.*'
//                             The files 'ABC.TXT', 'ABCDEFG.TXT',
//                             'AbCdEfG.tXt' are found.
//           mask_dir:         Indicator for comparing found
//                             directories against 'file_mask'.
//                             If set to *On, directories found will
//                             have their names compared to the file
//                             mask. If there is a match, the
//                             FileFound() callback procedure will be
//                             called.
//                             If set to *Off, directories will not be
//                             compared against the 'file_mask'. The
//                             FileFound() callback will be called for
//                             the directory.
//           search_subdirs:   Indicator for search of subfolders:
//                             *On  - Subfolders will be searched too.
//                             *Off - Only the folder in
//                                    starting_path will be searched.
//           pointer           This must be a procedure pointer to
//                             a procedure you write that will be
//                             called for each file/directory found.
//           )
//
// If FindFiles() returns *On, it means that the user supplied
// FileFound() function returned *On to terminate the search.
// Otherwise, FindFiles() returns *Off.
//
// Examples:
// To find all files with extension txt i /home and subdirs:
//   FindFiles('/home': '*.txt': *On: *On: %PAddr('MYFILEFOUND'))
//
// To find all objects of type *PGM in JWTOOLS:
//   FindFiles('/qsys.lib/jwtools.lib': '*.pgm': *Off: *Off:
//             %PAddr('MYFILEFOUND'))
// Please note, that when searching a library other that QSYS it is
// a waiste of CPU time to specify *On for subfolders.
//
// To find all *DTAARA in the QSYS file system:
//   FindFiles('/qsys.lib': '*.dtaara': *On: *Off:
//             %PAddr('MYFILEFOUND'))
// Here the subdir option is set to *ON because all libraries reside
// in QSYS.

